@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import configparser, os, subprocess
+import os, subprocess
 import typing as T
 
 from . import mesonlib
@@ -42,6 +42,7 @@ known_cpu_families = (
     'arm',
     'avr',
     'c2000',
+    'dspic',
     'e2k',
     'ia64',
     'm68k',
@@ -49,6 +50,7 @@ known_cpu_families = (
     'mips',
     'mips64',
     'parisc',
+    'pic24',
     'ppc',
     'ppc64',
     'riscv32',
@@ -57,10 +59,9 @@ known_cpu_families = (
     'rx',
     's390',
     's390x',
+    'sh4',
     'sparc',
     'sparc64',
-    'pic24',
-    'dspic',
     'wasm32',
     'wasm64',
     'x86',
@@ -82,33 +83,6 @@ CPU_FAMILES_64_BIT = [
     'x86_64',
 ]
 
-class MesonConfigFile:
-    @classmethod
-    def from_config_parser(cls, parser: configparser.ConfigParser) -> T.Dict[str, T.Dict[str, T.Dict[str, str]]]:
-        out = {}
-        # This is a bit hackish at the moment.
-        for s in parser.sections():
-            section = {}
-            for entry in parser[s]:
-                value = parser[s][entry]
-                # Windows paths...
-                value = value.replace('\\', '\\\\')
-                if ' ' in entry or '\t' in entry or "'" in entry or '"' in entry:
-                    raise EnvironmentException('Malformed variable name %s in cross file..' % entry)
-                try:
-                    res = eval(value, {'__builtins__': None}, {'true': True, 'false': False})
-                except Exception:
-                    raise EnvironmentException('Malformed value in cross file variable %s.' % entry)
-
-                for i in (res if isinstance(res, list) else [res]):
-                    if not isinstance(i, (str, int, bool)):
-                        raise EnvironmentException('Malformed value in cross file variable %s.' % entry)
-
-                section[entry] = res
-
-            out[s] = section
-        return out
-
 def get_env_var_pair(for_machine: MachineChoice,
                      is_cross: bool,
                      var_name: str) -> T.Tuple[T.Optional[str], T.Optional[str]]:
@@ -225,11 +199,11 @@ class MachineInfo:
 
         cpu_family = literal['cpu_family']
         if cpu_family not in known_cpu_families:
-            mlog.warning('Unknown CPU family %s, please report this at https://github.com/mesonbuild/meson/issues/new' % cpu_family)
+            mlog.warning('Unknown CPU family {}, please report this at https://github.com/mesonbuild/meson/issues/new'.format(cpu_family))
 
         endian = literal['endian']
         if endian not in ('little', 'big'):
-            mlog.warning('Unknown endian %s' % endian)
+            mlog.warning('Unknown endian {}'.format(endian))
 
         return cls(literal['system'], cpu_family, literal['cpu'], endian)
 
