@@ -6,6 +6,9 @@ interface
 module elemental real function pi2tau(pi)
   real, intent(in) :: pi
 end function pi2tau
+
+module subroutine good()
+end subroutine good
 end interface
 
 end module parent
@@ -17,4 +20,6 @@ tau = pi2tau(pi)
 
 print *,'pi=',pi, 'tau=', tau
 
+call good()
+
 end program
